function [newRPeaks, newBoundaries] = correctFalseNegRPks(rPeaks, RToQRSBoundary, phaseSignal, ampSignal)
% Given the rPeaks found from an ECG signal,
% it finds additional R-peaks in the RR-interval that is longer than %150
% of the preciding RR-interval
% rPeaks: currently detected R-peaks
% RToQRSBoundary: for each R-peak it gives the QRS boundary from
% thresholding the phaseSignal.
% phaseSignal: Phase signal from the phasor transform of the ECG signal.
% ampSignal: Amplitude signal from the phasor transform of the ECG signal.
% newRPeaks: Modified R-peak indices.
% rrIntervals = diff(rPeaks);
% falseNegativesInd = find(rrIntervals(2:end) > rrIntervals(1:end-1).*1.5)
% for i = 1:length(falseNegativesInd')
%     curIndex = falseNegativesInd(i);
%     leftIndex = RToQRSBoundary(curIndex+1,2);
%     rightIndex = RToQRSBoundary(curIndex+2,1);
%     isPeakDetected = false;
%     threshold = 0.003;
%     while (~isPeakDetected)
%         threshold = threshold + 0.0005;
%         segment = phaseSignal(leftIndex:rightIndex) > (pi/2 - threshold);
%         onOffIndex = find(diff(segment));
%         if (length(onOffIndex) >= 3)
%             onOffIndex
%             isPeakDetected = true;
%             [rPeakLoc, bound] = findRPeakFromSegment(segment, onOffIndex, ampSignal);
%             rPeakLoc = leftIndex + rPeakLoc;
%             bound = bound + leftIndex;
%             rPeaks = [rPeaks(1:curIndex+1); rPeakLoc; rPeaks(curIndex+2:end)];
%             RToQRSBoundary = [RToQRSBoundary(1:curIndex+1,:); bound; ...
%                 RToQRSBoundary(curIndex+2:end,:)];
%             falseNegativesInd = falseNegativesInd + 1;
%         elseif (threshold > pi / 2)
%             isPeakDetected = true;
%             fprintf('No Peak\n')
%         end
%     end
% end
% newRPeaks = rPeaks;
% newBoundaries = RToQRSBoundary;
rrIntervals = diff(rPeaks);
falseNegativesInd = find(rrIntervals(2:end) > rrIntervals(1:end-1).*1.5);
newRPeaks = zeros(length(rPeaks) + nnz(falseNegativesInd).*3, 1);
newBoundaries = zeros(length(rPeaks) + nnz(falseNegativesInd).*3, 2);
newRPeaks(1:3) = rPeaks(1:3);
newBoundaries(1:3,:) = RToQRSBoundary(1:3,:);
oldRPeakCounter = 3;
beatCounter = 2;
newInterval = rPeaks(3)-rPeaks(2);
oldInterval = rPeaks(2)-rPeaks(1);
while (oldRPeakCounter <= length(rPeaks))
    if (newInterval > (oldInterval*1.5))
        leftIndex = newBoundaries(beatCounter, 2);
        rightIndex = newBoundaries(beatCounter + 1, 1);
        isPeakDetected = false;
        threshold = 0.003;
        while (~isPeakDetected)
            threshold = threshold + 0.0005;
            segment = phaseSignal(leftIndex:rightIndex) > (pi/2 - threshold);
            onOffIndex = find(diff(segment));
            if (length(onOffIndex) >= 3)
                isPeakDetected = true;
                [rPeakLoc, bound] = findRPeakFromSegment(segment, onOffIndex, ampSignal);
                rPeakLoc = leftIndex + rPeakLoc;
                bound = bound + leftIndex;
                beatCounter = beatCounter + 1;
                newRPeaks(beatCounter) = rPeakLoc;
                newBoundaries(beatCounter,:) = bound;z
                newInterval = rPeaks(oldRPeakCounter) - rPeakLoc;
                oldInterval = rPeakLoc - newRPeaks(beatCounter - 1);
            elseif (threshold > pi/2)
                isPeakDetected = true;
                fprintf('No Peak\n')
            end
        end
    else
        beatCounter = beatCounter + 1;
        newRPeaks(beatCounter) = rPeaks(oldRPeakCounter);
        newBoundaries(beatCounter,:) = RToQRSBoundary(oldRPeakCounter, :);
        oldRPeakCounter = oldRPeakCounter + 1;
        if (oldRPeakCounter <= length(rPeaks))
            newRPeaks(beatCounter + 1) = rPeaks(oldRPeakCounter);
            newBoundaries(beatCounter + 1) = rPeaks(oldRPeakCounter);
            newInterval = rPeaks(oldRPeakCounter) - newRPeaks(beatCounter);
            oldInterval = newRPeaks(beatCounter) - newRPeaks(beatCounter - 1);
        end
    end
end
newRPeaks = newRPeaks(1:beatCounter);
newBoundaries = newBoundaries(1:beatCounter,:);
end