function [newRPeaks, newBoundaries, removedRpksInd] = correctFalsePosRPks(rPeaks, RToQRSBoundary, ampSignal)
% Given the rPeaks found from an ECG signal,
% this code detects false positive r-peaks in the following manner:
% if the RR interval is %40 shorter than the preceding RR interval, then
% one of the R-peaks in the RR interval is decleared false positive.
% From the two potential false positive R-peak, the one with higher
% absolute magnitude is kept.
% rPeaks: currently detected R-peaks
% RToQRSBoundary: for each R-peak it gives the QRS boundary from
% thresholding the phaseSignal.
% ampSignal: Amplitude signal from the phasor transform of the ECG signal.
% newRPeaks: Modified R-peak indices.

%% Detect Beats that include false positive R-peaks.
newRPeaks = rPeaks;
newBoundaries = RToQRSBoundary;
rrIntervals = diff(rPeaks);
beatCounter = 2;
removedRpksInd = zeros(100,1);
removeCounter = 0;
while (beatCounter <= length(rrIntervals))
    if (rrIntervals(beatCounter) < (rrIntervals(beatCounter-1)*0.4))
        removeCounter = removeCounter + 1;
        leftRPeak = newRPeaks(beatCounter); 
        rightRPeak = newRPeaks(beatCounter + 1);
        if (ampSignal(leftRPeak) > ampSignal(rightRPeak))
            newRPeaks(beatCounter + 1) = [];
            newBoundaries(beatCounter + 1, :) = [];
            removedRpksInd(removeCounter) = beatCounter + removeCounter;
        else
            newRPeaks(beatCounter) = [];
            newBoundaries(beatCounter, :) = [];
            removedRpksInd(removeCounter) = beatCounter + removeCounter - 1;
        end
        rrIntervals = diff(newRPeaks);
    else
        beatCounter = beatCounter + 1;
    end
end
removedRpksInd = removedRpksInd(1:removeCounter);
end

%     while (~isPeakDetected)
%         threshold = threshold + 0.0005;
%         segment = phaseSignal(leftIndex:rightIndex) > (pi/2 - threshold);
%         onOffIndex = find(diff(segment));
%         if (length(onOffIndex) >= 3)
%             onOffIndex
%             isPeakDetected = true;
%             [rPeakLoc, bound] = findRPeakFromSegment(segment, onOffIndex, ampSignal);
%             rPeakLoc = leftIndex + rPeakLoc;
%             bound = bound + leftIndex;
%             rPeaks = [rPeaks(1:curIndex+1); rPeakLoc; rPeaks(curIndex+2:end)];
%             RToQRSBoundary = [RToQRSBoundary(1:curIndex+1,:); bound; ...
%                 RToQRSBoundary(curIndex+2:end,:)];
%             falseNegativesInd = falseNegativesInd + 1;
%         elseif (threshold > pi / 2)
%             isPeakDetected = true;
%             fprintf('No Peak\n')
%         end
%     end