function [newRPeaks] = BurakCorrectRPks2(rPeaks, phaseSignal, ampSignal, signal, fs)
% Given the rPeaks found from an ECG signal, it corrects rPeaks based on
% the paper.
% rPeaks: currently detected R-peaks
% RToQRSBoundary: for each R-peak it gives the QRS boundary from
% thresholding the phaseSignal.
% phaseSignal: Phase signal from the phasor transform of the ECG signal.
% ampSignal: Amplitude signal from the phasor transform of the ECG signal.
% newRPeaks: Modified R-peak indices.
% global recName;
% global i;
% global problemWaves;
% global problemCounter;
refPeriod = 0.25*fs; % 250ms refractory period
isFigure = false;
if (isFigure)
    f1 = figure;
    f2 = figure; % Loop Check
end
rrIntervals = diff(rPeaks);
falseNegativesInd = find(rrIntervals(2:end) > rrIntervals(1:end-1).*1.5);
newRPeaks = zeros(length(rPeaks) + nnz(falseNegativesInd).*3, 1);
newBoundaries = zeros(length(rPeaks) + nnz(falseNegativesInd).*3, 2);
rrIntervals = zeros(length(newRPeaks),1);
newRPeaks(1:3) = rPeaks(1:3);
rrIntervals(1) = newRPeaks(2)-newRPeaks(1);
rrIntervals(2) = newRPeaks(3)-newRPeaks(2);
rrBuffer = zeros(5,1);
boundBuffer = zeros(5,2);
thBuffer = zeros(5,1);
bufferCounter = 0;
oldRPeakCounter = 4;
rPeakCounter = 2;
intCounter = 2;
isMoveOneBeat = false;
while (oldRPeakCounter <= length(rPeaks) + 1)
    if (rrIntervals(intCounter) < (rrIntervals(intCounter-1)*0.45) || ...
            rrIntervals(intCounter) <= refPeriod)
        %if the current RR-interval is less than 40% of the previous
        %RR-interval
        leftRPeak = newRPeaks(rPeakCounter);
        rightRPeak = newRPeaks(rPeakCounter + 1);
        isKeepLeft = ampSignal(leftRPeak) > ampSignal(rightRPeak);
        if (~isKeepLeft)
            newRPeaks(rPeakCounter) = newRPeaks(rPeakCounter + 1);
            newBoundaries(rPeakCounter,:) = newBoundaries(rPeakCounter + 1,:);
        end
        if (bufferCounter == 0)
            if (oldRPeakCounter <= length(rPeaks))
                newRPeaks(rPeakCounter + 1) = rPeaks(oldRPeakCounter);
            else
                rPeakCounter = rPeakCounter - 1;
            end
            oldRPeakCounter = oldRPeakCounter + 1;
        else
            newRPeaks(rPeakCounter + 1) = rrBuffer(bufferCounter);
            newBoundaries(rPeakCounter + 1,:) = boundBuffer(bufferCounter,:);
            bufferCounter = bufferCounter - 1;
            rrIntervals(intCounter) = newRPeaks(rPeakCounter+1) - newRPeaks(rPeakCounter);
            rrIntervals(intCounter-1) = newRPeaks(rPeakCounter) - newRPeaks(rPeakCounter - 1);
            if (rrIntervals(intCounter) < (rrIntervals(intCounter-1)*0.45))
                isMoveOneBeat = false;
            else
                isMoveOneBeat = true;
            end
        end
        
    elseif (rrIntervals(intCounter) > (rrIntervals(intCounter-1)*1.6))
        % if the current RR-interval is greater than 150% of the previous
        % RR-interval
        leftIndex = newBoundaries(rPeakCounter, 2);
        rightIndex = newBoundaries(rPeakCounter + 1, 1) + 1;
        [newRPeakLoc, bound] = BurakFindRPeakFromSegment(phaseSignal, ...
            ampSignal, leftIndex, rightIndex, fs);
        if (newRPeakLoc ~= 0)
            bufferCounter = bufferCounter + 1;
            
            if (bufferCounter > 5)
%                 fprintf('Problem in rec: %s\n', recName)
                fprintf('There is likely a problem\n')
                %                 problemCounter = problemCounter + 1;
                %                 problemWaves(problemCounter) = i;
                newRPeaks = rPeaks;
                return
            end
            rrBuffer(bufferCounter) = newRPeaks(rPeakCounter + 1);
            boundBuffer(bufferCounter,:) = newBoundaries(rPeakCounter + 1,:);
            newRPeaks(rPeakCounter + 1) = newRPeakLoc;
            newBoundaries(rPeakCounter + 1,:) = bound;
            if (bufferCounter == 1 && isFigure)
                figure(f1)
                xx = 1:length(signal);
                plot(signal)
                hold on
                plot(xx(newRPeaks(1:rPeakCounter+1)), signal(newRPeaks(1:rPeakCounter+1)), 'ro')
                plot(rrBuffer(1), signal(rrBuffer(1)), 'ro')
                asd = [newRPeaks(1:rPeakCounter + 1); rrBuffer(1)];
                xPos = asd(1:end-1) + diff(asd)./2;
                inttt = diff(asd);
                meanLevel = mean(signal);
                stdDev = std(signal);
                for k = 1:2:length(xPos)
                    text(xPos(k), meanLevel+2*stdDev, char(string(inttt(k))));
                end
                for k = 2:2:length(xPos)
                    text(xPos(k), meanLevel-2*stdDev, char(string(inttt(k))));
                end
                for k = 1:length(asd)
                    text(asd(k), signal(asd(k))+stdDev, char(string(asd(k))));
                end
                hold off
            elseif (isFigure)
                hold on
                asd = [newRPeaks(rPeakCounter+1); flipud(rrBuffer(1:bufferCounter))];
                xPos = asd(1:end-1) + diff(asd)./2;
                intt = diff(asd);
                for k = 1:2:length(xPos)
                    text(xPos(k), meanLevel+2*stdDev, char(string(intt(k))));
                end
                for k = 2:2:length(xPos)
                    text(xPos(k), meanLevel-2*stdDev, char(string(intt(k))));
                end
                for k = 1:length(asd)
                    text(asd(k), signal(asd(k))+stdDev, char(string(asd(k))));
                end
                plot(rrBuffer(1:bufferCounter), signal(rrBuffer(1:bufferCounter)), 'ro')
                hold off
            end
        else
            isMoveOneBeat = true;
        end
        
    else
        isMoveOneBeat = true;
    end
   
    
    if (isMoveOneBeat)
        %No problem with the current RR-interval so advance one R peak
        rPeakCounter = rPeakCounter + 1;
        intCounter = intCounter + 1;
        %Add the new r peak to r-peak list
        if (bufferCounter == 0)
            if (oldRPeakCounter <= length(rPeaks))
                newRPeaks(rPeakCounter + 1) = rPeaks(oldRPeakCounter);
            else
                rPeakCounter = rPeakCounter - 1;
            end
            oldRPeakCounter = oldRPeakCounter + 1;
        else
            newRPeaks(rPeakCounter + 1) = rrBuffer(bufferCounter);
            newBoundaries(rPeakCounter + 1, :) = boundBuffer(bufferCounter, :);
            bufferCounter = bufferCounter - 1;
        end
        isMoveOneBeat = false;
    end
    rrIntervals(intCounter) = newRPeaks(rPeakCounter+1) - newRPeaks(rPeakCounter);
    if (rPeakCounter > 1)
        rrIntervals(intCounter-1) = newRPeaks(rPeakCounter) - newRPeaks(rPeakCounter - 1);
    end
    if (isFigure)
        figure(f2)
        plot(signal)
        hold on
        plot(newRPeaks(1:rPeakCounter+1), signal(newRPeaks(1:rPeakCounter+1)), 'ro')
        hold off
    end
end
% if (rrIntervals(intCounter) < (rrIntervals(intCounter-1)*0.45))
%     %if the current RR-interval is less than 40% of the previous
%     %RR-interval
%     leftRPeak = newRPeaks(rPeakCounter);
%     rightRPeak = newRPeaks(rPeakCounter + 1);
%     isKeepLeft = ampSignal(leftRPeak) > ampSignal(rightRPeak);
%     if (~isKeepLeft)
%         newRPeaks(rPeakCounter) = newRPeaks(rPeakCounter + 1);
%         newBoundaries(rPeakCounter,:) = newBoundaries(rPeakCounter + 1,:);
%     end
%     if (bufferCounter == 0)
%         newRPeaks(rPeakCounter + 1) = rPeaks(oldRPeakCounter);
%         newBoundaries(rPeakCounter + 1,:) = RToQRSBoundary(oldRPeakCounter,:);
%         oldRPeakCounter = oldRPeakCounter + 1;
%     else
%         newRPeaks(rPeakCounter + 1) = rrBuffer(bufferCounter);
%         newBoundaries(rPeakCounter + 1,:) = boundBuffer(bufferCounter,:);
%         bufferCounter = bufferCounter - 1;
%         rrIntervals(intCounter) = newRPeaks(rPeakCounter+1) - newRPeaks(rPeakCounter);
%         rrIntervals(intCounter-1) = newRPeaks(rPeakCounter) - newRPeaks(rPeakCounter - 1);
%         isMoveOneBeat = true;
%     end
%
% elseif (rrIntervals(intCounter) > (rrIntervals(intCounter-1)*1.6))
%     % if the current RR-interval is greater than 150% of the previous
%     % RR-interval
%     leftIndex = newBoundaries(rPeakCounter, 2);
%     rightIndex = newBoundaries(rPeakCounter + 1, 1) + 1;
%     [newRPeakLoc, bound] = findRPeakFromSegment(phaseSignal, ...
%         ampSignal, leftIndex, rightIndex, fs);
%     if (newRPeakLoc ~= 0)
%         bufferCounter = bufferCounter + 1;
%         if (bufferCounter > 5)
%             fprintf('There is likely a problem\n')
%         end
%         rrBuffer(bufferCounter) = newRPeaks(rPeakCounter + 1);
%         boundBuffer(bufferCounter,:) = newBoundaries(rPeakCounter + 1,:);
%         newRPeaks(rPeakCounter + 1) = newRPeakLoc;
%         newBoundaries(rPeakCounter + 1,:) = bound;
%     else
%         isMoveOneBeat = true;
%     end
% end

newRPeaks = newRPeaks(1:rPeakCounter+1);
isFigure = false;
if (isFigure)
    figure
    xx = 1:length(signal);
    plot(signal)
    hold on
    plot(xx(newRPeaks(1:rPeakCounter+1)), signal(newRPeaks(1:rPeakCounter+1)), 'ro')
    hold off
end
end