function [rPeakLoc, bound] = BurakFindRPeakFromSegment(phaseSignal, ampSignal, leftIndex, rightIndex, fs)
% Finds an R-peak in the given interval [leftIndex, rightIndex] based
% on lowering threshold.
% phaseSignal: phase signal from the phasor transform of the ECG
% ampSignal: amplitude signal from the phasor transform of the ECG
% leftIndex: start of the segment
% rightIndex: end of the segment
% fs: sampling frequency of the ecg
if ((rightIndex-leftIndex)/fs < 0.2) %if the interval is less than 200ms.
    rPeakLoc = 0;
    bound = 0;
    return
end
threshold = 0.003;
isPeakDetected = false;
while (~isPeakDetected)
    threshold = threshold + 0.0005;
    segment = phaseSignal(leftIndex:rightIndex) > (pi/2-threshold);
    onOffIndex = find(diff(segment));
    if (length(onOffIndex) >= 3 && mod(length(onOffIndex),2) == 0)
        isPeakDetected = true;
        %[rPeakLoc, bound] = findRPeakFromSegment(segment, onOffIndex, ampSignal);
        onOffIndex = onOffIndex(2:end-1);
        maxAmp = -1;
        for l = 1:2:length(onOffIndex)
            start = onOffIndex(l) + 1;
            endS = onOffIndex(l+1);
            [curAmp, curRIndex] = max(ampSignal(start:endS));
            if (curAmp > maxAmp)
                maxAmp = curAmp;
                rPeakLoc = start + curRIndex;
                bound = [start, endS];
            end
        end
        rPeakLoc = leftIndex + rPeakLoc - 1;
        bound = bound + leftIndex;
    elseif (threshold > pi/2)
        rPeakLoc = 0;
        bound = 0;
        isPeakDetected = true;
        fprintf('No Peak\n')
    end
end
% offSet = 5;
% [~, ind] = max(ampSignal(rPeakLoc-offSet:rPeakLoc+offSet));
% newRPeakLoc = 

end
