function [fidInds] = BurakSegtoDel(phaseSignal, ampSignal, signal,fs, isFigure)
%Delineation of the ECG segment
%phaseSignal: Phase signal of the phasor transform.
%ampSignal: Amplitude signal of the phasor transofrm.
%fidInds: Indices of the fiducial points.
%% Preperation
if (isFigure)
    persistent f1 %Visualize R peak detection
    persistent f2 %Figure to visualize Q and S detection process.
    persistent f3 %Figure to visualize Q and S detection process.
    persistent f4 %Unused
    if (isempty(f1) || ~ishandle(f1))
        f1 = figure;
    end
    if (isempty(f2) || ~ishandle(f2))
        f2 = figure;
    end
    if (isempty(f3) || ~ishandle(f3))
        %         f3 = figure;
    end
    if (isempty(f4) || ~ishandle(f4))
        f4 = figure;
    end
end
%% ECG signal thresholding
threshold = 0.003;
QRSSegments = phaseSignal > (pi/2 - threshold);
%% QRS Segments to R-peaks and R-peaks to QRS boundaries
onOffSetIndex = find(diff(QRSSegments));
if (isempty(onOffSetIndex))
    fidInds = 0;
    return
end
if (QRSSegments(1) ~= 1)
    numRpeaks = ceil(length(onOffSetIndex)/2);
else
    numRpeaks = ceil((length(onOffSetIndex) + 1)/2);
end
rPeaksInd = zeros(numRpeaks,1);
RToQRSBoundaries = zeros(numRpeaks,2);
iter = 1;
if (QRSSegments(1) == 1)
    [~, rPeaksInd(1)] = max(ampSignal(1:onOffSetIndex(1)));
    RToQRSBoundaries(1,:) = [1, onOffSetIndex(1)];
    onOffSetIndex(1) = [];
    iter = 2;
end
for l = 1:2:length(onOffSetIndex)
    leftIndex = onOffSetIndex(l) + 1;
    if (l + 1 <= length(onOffSetIndex))
        rightIndex = onOffSetIndex(l+1);
    else
        rightIndex = length(ampSignal);
    end
    [~, localOpt] = max(ampSignal(leftIndex:rightIndex));
    rPeaksInd(iter) = leftIndex + localOpt - 1;
    RToQRSBoundaries(iter,:) = [leftIndex, rightIndex];
    iter = iter + 1;
end

%% If the first beat is erroneous, fix it.
while (length(rPeaksInd) >= 2 && (rPeaksInd(2)-rPeaksInd(1))/fs < 250e-3)
    leftR = phaseSignal(rPeaksInd(1));
    rightR = phaseSignal(rPeaksInd(2));
    if (leftR > rightR)
        rPeaksInd(2) = [];
        RToQRSBoundaries(2,:) = [];
    else
        rPeaksInd(1) = [];
        RToQRSBoundaries(1,:) = [];
    end
end

%% If the bpm is < 30, decleare an erroneous waveform.
mins = length(signal)/fs/60;
bpm = (length(rPeaksInd)-1)/mins;
rrInterval = diff(rPeaksInd);
if (bpm < 20 || any(rrInterval > 2*fs) || rPeaksInd(1) > 2*fs ||...
        (length(signal)-rPeaksInd(end)) > 2*fs)
    fidInds = 0;
    return
end

%% R peak correction based on the paper
newRPeaksInd = BurakCorrectRPks(rPeaksInd, RToQRSBoundaries, phaseSignal, ampSignal, signal, fs);
% if (isFigure)
%     figure(fHandle(1))
%     plot(signal)
%     hold on
%     plot(rPeaksInd, signal(rPeaksInd) + 2, 'ko')
%     plot(newRPeaksInd, signal(newRPeaksInd), 'ro')
%     %     plot(rPeaksInd, signal(rPeaksInd), 'ko')
%     hold off
% end

mins = length(signal)/fs/60;
bpm = (length(rPeaksInd)-1)/mins;
rrInterval = diff(rPeaksInd);
if (bpm < 20 || bpm > 220 || any(rrInterval > 2.5*fs) || rPeaksInd(1) > 2.5*fs ||...
        (length(signal)-rPeaksInd(end)) > 2.5*fs)
    fidInds = 0;
    return
end
rPeaksInd = newRPeaksInd;
if (sum(sign(signal(rPeaksInd))) > 0)
    waveSign = 1;
else
    waveSign = -1;
end
pols = sign(signal(rPeaksInd));
% meanRPeakVal = mean(signal(rPeaksInd));
% stdRPeakVal = std(signal(rPeaksInd));
% if (meanRPeakVal > mean(signal))
%     waveSign = 1;
% else
%     waveSign = -1;
% end
% pols = zeros(length(rPeaksInd),1);
% for i = 1:length(pols)
%    curR = rPeaksInd(i);
%    if (waveSign == 1)
%        if (signal(curR) < meanRPeakVal-2.5*stdRPeakVal)
%            pols(i) = -1;
%        else
%            pols(i) = 1;
%        end
%    else
%        if (signal(curR) > meanRPeakVal+2.5*stdRPeakVal)
%            pols(i) = 1;
%        else
%            pols(i) = -1;
%        end
%    end
% end
%% Visualize Signal
% Detrend the signal
if (isFigure && ishandle(f1))
    figure(f1)
    subplot(2,1,1)
    plot(signal)
    hold on
    plot(rPeaksInd, signal(rPeaksInd), 'ro')
    plot(rPeaksInd, pols, 'go')
    hold off
    subplot(2,1,2)
    plot(phaseSignal)
    hold on
    plot(rPeaksInd, phaseSignal(rPeaksInd), 'ro')
    hold off
    
end
%% QRS Delineation

if (waveSign == -1)
    newSignal = waveSign.*signal;
else
    newSignal = signal;
end

QInds = zeros(length(rPeaksInd),1);
SInds = zeros(length(rPeaksInd),2);
% first column max and second column offset.
PInds = zeros(length(rPeaksInd),3);
%First column is P peak, second one is onset and third one is the offset index
TInds = zeros(length(rPeaksInd),3);
%First column is T peak, second one is onset and third one is the offset index
for i = 1:length(rPeaksInd)
    if (i == 1)
        QInds(i) = NaN;
        SInds(i,1) = NaN;
        SInds(i,2) = NaN;
    elseif (i == length(rPeaksInd))
        QInds(i) = NaN;
        SInds(i,1) = NaN;
        SInds(i,2) = NaN;
    else
        sigToCons = newSignal(rPeaksInd(i-1):rPeaksInd(i+1));
        rPeakPos = rPeaksInd(i)-rPeaksInd(i-1)+1;
        windowLen = round(35*1e-3*fs); %35ms window
        
        % Find Q
        [gammaNeg, isQValid] = getGamma(phaseSignal(rPeaksInd(i-1):rPeaksInd(i+1)), rPeakPos, 'Q');
        if (isQValid)
            winStart = gammaNeg-windowLen;
            winEnd = gammaNeg;
            posSig = sigToCons(winStart:winEnd) - median(sigToCons(winStart:winEnd));
            [newPhaseSig, newAmpSig] = BurakPTransform(posSig, 0.005);
            if (isFigure && ~isempty(f3))
                figure(f3)
                subplot(3,1,1)
                plot(posSig)
                subplot(3,1,2)
                plot(newPhaseSig)
                subplot(3,1,3)
                plot(newAmpSig)
            end
            %         [posPhases, Qind] = findpeaks(-1.*newPhaseSig);
            [~, locMins] = findpeaks(-1.*newPhaseSig);
            if (isempty(locMins))
                qPIndex = winStart;
            else
                qPIndex = winStart + locMins(1) - 1;
            end
            QInds(i) = qPIndex + rPeaksInd(i-1) - 1;
            
        else
            QInds(i) = NaN;
        end
        
        
        % Find S
        [gammaPos, isSValid] = getGamma(phaseSignal(rPeaksInd(i-1):rPeaksInd(i+1)), rPeakPos, 'S');
        if (isSValid)
            gammaPos = gammaPos + rPeaksInd(i) - rPeaksInd(i-1);
            winStart = gammaPos;
            winEnd = gammaPos + windowLen;
            posSig = sigToCons(winStart:winEnd) - median(sigToCons(winStart:winEnd));
            [newPhaseSig, newAmpSig] = BurakPTransform(posSig, 0.005);
            [~, locMins] = findpeaks(-1.*newPhaseSig);
            if (isempty(locMins))
                sPIndex = winEnd;
            else
                sPIndex = winStart + locMins(end) - 1;
            end
            SInds(i,2) = sPIndex + rPeaksInd(i-1) - 1;
            [~, maxInd] = max(abs(sigToCons(winStart:winEnd)));
            sPIndex2 = maxInd + winStart - 1;
            SInds(i,1) = maxInd + winStart + rPeaksInd(i-1) - 2;
            %             candidatePoints = newPhaseSig > max(newPhaseSig)*0.5;
            %             if nnz(candidatePoints) == 1
            %                 sPoint = find(candidatePoints);
            %             else
            %                 [~, maxInd] = max(newAmpSig(candidatePoints));
            %                 sPoint = find(candidatePoints);
            %                 sPoint = sPoint(maxInd);
            %             end
            %             sPIndex = winStart + sPoint - 1;
        else
            SInds(i,1) = NaN;
            SInds(i,2) = NaN;
        end
        
        
        
        
        % Find P
        isPDetected = false;
        winDec = round(fs*30e-3);
        windowLen = round(0.25.*(rPeaksInd(i)-rPeaksInd(i-1))) + winDec;
        windowEnd = qPIndex - round(fs*20e-3);
        pLoc = NaN;
        while (~isPDetected && (windowLen/fs)>35e-3) %Possible optimization here. No Need to recalculate phasor transform in each iteration of the loop.
            windowLen = windowLen - winDec;
            windowStart = windowEnd-windowLen;
            newSig = sigToCons(windowStart:windowEnd)-median(sigToCons(windowStart:windowEnd));
            isoLevel = median(sigToCons(windowStart:windowEnd));
            stdLevel = std(sigToCons(windowStart:windowEnd));
            [phaseSig, ampSig] = BurakPTransform(newSig, 0.005);
            if (isFigure && ~isempty(f4))
                figure(f4)
                subplot(3,1,1)
                plot(sigToCons(windowStart:windowEnd))
                subplot(3,1,2)
                plot(newSig)
                subplot(3,1,3)
                plot(phaseSig)
            end
            [maxPhases, maxInd] = findpeaks(phaseSig, 'MinPeakProminence', max(phaseSig)/2);
            if (~isempty(maxInd))
                [~, indIndMax] = max(maxPhases);
                pLoc = maxInd(indIndMax) + windowStart - 1;
                isPDetected = true;
            else
                [maxPhases, maxInd] = findpeaks(phaseSig);
                if (~isempty(maxInd))
                    pLoc = maxInd(1) + windowStart - 1;
                    isPDetected = true;
                end
            end
            %             [~, maxInd] = max(phaseSig);
            %             if (maxInd == length(phaseSig))
            %                 pLoc = NaN;
            %                 isPDetected = true;
            %             elseif (maxInd ~= 1)
            %                 if (ampSig(maxInd) > ampSig(maxInd-1) && ampSig(maxInd) > ampSig(maxInd+1))
            %                     isPDetected = true;
            %                     pLoc = maxInd + windowStart - 1;
            %                 end
            %             end
        end
        if (~isnan(pLoc))
            if (true)%(sigToCons(pLoc) > 0*sigToCons(rPeakPos))
                [~, minInds] = findpeaks(-1.*phaseSig, 'MinPeakProminence', max(phaseSig)/3);
                pOnInd = find((minInds + windowStart - 1 < pLoc), 1, 'last');
                if (isempty(pOnInd))
                    pOnset = windowStart;
                else
                    pOnset = minInds(pOnInd) + windowStart - 1;
                end
                pOffInd = find((minInds + windowStart - 1) > pLoc, 1, 'first');
                if (isempty(pOffInd))
                    pOffset = windowEnd;
                else
                    pOffset = minInds(pOffInd) + windowStart - 1;
                end
                
                %                   numZeros = length(minInds);
                %                   minInds = [1, minInds];
                %                   [~, zeroCross] = min(abs(minInds-pPhase));
                %                   if (zeroCross+1 > numZeros)
                %                       pOnset = minInds(zeroCross - 1) + windowStart - 1;
                %                       pOffset = minInds(zeroCross) + windowStart - 1;
                %                   else
                %                       if (abs(minInds(zeroCross-1)-pPhase) > abs(minInds(zeroCross+1)-pPhase))
                %                           pOnset = minInds(zeroCross) + windowStart - 1;
                %                           pOffset = minInds(zeroCross + 1) + windowStart - 1;
                %                       else
                %                           pOnset = minInds(zeroCross - 1) + windowStart - 1;
                %                           pOffset = minInds(zeroCross) + windowStart - 1;
                %                       end
                %                   end
                
                
                %                 [newPhaseSig] = BurakPTransform(newSig, 0.005);
                %                 if (isFigure && ~isempty(f4))
                %                     figure(f4)
                %                     subplot(3,1,1)
                %                     plot(sigToCons(winStart:winEnd))
                %                     subplot(3,1,2)
                %                     plot(sigToCons(winStart:winEnd) - median(sigToCons(winStart:winEnd)))
                %                     subplot(3,1,3)
                %                     plot(newPhaseSig)
                %                 end
                %                 [amps, locs] = findpeaks(diff(newPhaseSig));
                %                 [~, indMax] = max(amps);
                %                 pOnset = locs(indMax);
                %                 pOnset = pOnset + winStart - 1;
                %                 winStart = pLoc;
                %                 winEnd = pLoc+winLen;
                %                 newSig = sigToCons(winStart:winEnd) - median(sigToCons(winStart:winEnd));
                %                 [newPhaseSig] = BurakPTransform(newSig, 0.005);
                %                 [amps, locs] = findpeaks(-1.*diff(newPhaseSig));
                %                 [~, indMax] = max(amps);
                %                 pOffset = locs(indMax);
                %                 pOffset = pOffset + winStart - 1;
            end
        end
        pInds(i, 1) = pLoc + rPeaksInd(i-1) - 1;
        pInds(i, 2) = pOnset + rPeaksInd(i-1) - 1;
        pInds(i, 3) = pOffset + rPeaksInd(i-1) - 1;
        
        % Find T
        isTDetected = false;
        winInc = round(fs*30e-3);
        windowLen = round(0.4.*(rPeaksInd(i+1)-rPeaksInd(i))) - winInc;
        lastCons = round(0.6.*(rPeaksInd(i+1)-rPeaksInd(i)));
        windowStart = sPIndex + round(fs*50e-3);
        tLoc = NaN;
        while (~isTDetected && (windowLen)<lastCons && windowEnd < length(sigToCons)) %Possible optimization here. No Need to recalculate phasor transform in each iteration of the loop.
            windowLen = windowLen + winInc;
            windowEnd = windowStart+windowLen;
            newSig = sigToCons(windowStart:windowEnd)-median(sigToCons(windowStart-round(fs*50e-3):windowEnd));
            isoLevel = median(sigToCons(windowStart:windowEnd));
            stdLevel = std(sigToCons(windowStart:windowEnd));
            [phaseSig, ampSig] = BurakPTransform(newSig, 0.1);
            [maxPhases, maxInd] = findpeaks(phaseSig, 'MinPeakProminence', max(phaseSig)/2);
            if (~isempty(maxInd))
                [~, indIndMax] = max(maxPhases);
                tLoc = maxInd(indIndMax) + windowStart - 1;
                isTDetected = true;
            else
                fprintf('problem\n')
            end
            %             [~, maxInd] = max(phaseSig);
            %             if (maxInd ~= length(phaseSig) && maxInd ~= 1)
            %                 if (ampSig(maxInd) > ampSig(maxInd-1) && ampSig(maxInd) > ampSig(maxInd+1))
            %                     isTDetected = true;
            %                     tLoc = maxInd + windowStart - 1;
            %                 end
            %             end
        end
        
        if (~isnan(tLoc))
            if (true)%(sigToCons(pLoc) > 0*sigToCons(rPeakPos))
                [~, minInds] = findpeaks(-1.*phaseSig, 'MinPeakProminence', max(phaseSig)/20);%, 'MinPeakProminence', max(phaseSig)/4);
                tOnInd = find((minInds + windowStart - 1 < tLoc), 1, 'last');
                if (isempty(pOnInd))
                    tOnset = windowStart;
                else
                    tOnset = minInds(tOnInd) + windowStart - 1;
                end
                tOffInd = find((minInds + windowStart - 1) > tLoc, 1, 'first');
                if (isempty(tOffInd))
                    tOffset = windowEnd;
                else
                    tOffset = minInds(tOffInd) + windowStart - 1;
                end
            end
        end
        
        pInds(i, 1) = tLoc + rPeaksInd(i-1) - 1;
        pInds(i, 2) = tOnset + rPeaksInd(i-1) - 1;
        pInds(i, 3) = tOffset + rPeaksInd(i-1) - 1;
        
        if (isFigure && ishandle(f2) && ~isempty(f2) && isSValid && isQValid)
            figure(f2)
            vecToPlot = signal(rPeaksInd(i-1):rPeaksInd(i+1));
            plot(vecToPlot)
            hold on
            plot(sPIndex, vecToPlot(sPIndex), 'ko', 'DisplayName', 's')
            plot(sPIndex2, vecToPlot(sPIndex2), 'yo', 'DisplayName', 's2')
            %             plot(winStart, vecToPlot(winStart), 'ro')
            %             plot(winEnd, vecToPlot(winEnd), 'ro')
            winEnd = qPIndex;
            windowLen = round(0.25.*(rPeaksInd(i)-rPeaksInd(i-1)));
            winStart = winEnd-windowLen;
            %             sigVal = median(vecToPlot(winStart:winEnd));
            %             asdasd = round((winStart+winEnd)/2);
            plot(qPIndex, vecToPlot(qPIndex), 'go', 'DisplayName', 'q')
            %             plot(asdasd, sigVal, 'ro', 'DisplayName', 'Median Value')
            %             pInd = pLoc - (rPeaksInd(i)-rPeakLoc)
            if (~isnan(pLoc))
                plot(pLoc, vecToPlot(pLoc), 'ro', 'DisplayName', 'P')
                plot(pOffset, vecToPlot(pOffset), 'r.','DisplayName', 'Poffset')
                plot(pOnset, vecToPlot(pOnset), 'r.','DisplayName', 'Ponnset')
            end
            if (~isnan(tLoc))
                plot(tLoc, vecToPlot(tLoc), 'co', 'DisplayName', 'T')
                plot(tOffset, vecToPlot(tOffset), 'c.', 'DisplayName', 'Toffset')
                plot(tOnset, vecToPlot(tOnset), 'c.', 'DisplayName', 'TOnset')
            end
            hold off
            legend('show')
        end
        
        
        
    end %end if
    
    
    
    
    %     windowLen = round(150e-3*fs);
    %     windowEnd = rPeaksInd(i)-1;
    %     windowStart = windowEnd-windowLen;
    %     derSignal = diff(signal(windowStart:windowEnd));
    %     sigToPlot = signal(windowStart:windowEnd);
    %     indNeg = find(sign(sigToPlot) == -1, 1, 'last');
    %     indNegDer = find(sign(derSignal) == -1, 1, 'last');
    %     if (isFigure && ishandle(f2) && ~isempty(f2))
    %        figure(f2)
    %        subplot(2,1,1)
    %        plot(sigToPlot)
    %        hold on
    %        plot(indNeg, sigToPlot(indNeg), 'ro')
    %        hold off
    %        subplot(2,1,2)
    %        plot(derSignal)
    %        hold on
    %        plot(indNegDer, derSignal(indNegDer), 'ro')
    %        hold off
    %     end
    
    
    
    
end
fidInds = 0;

    function [gammaToReturn, isValid] = getGamma(phaseSeg, rPeakLoc, QorS)
        timeWindowToLookFor = 75e-3; %75 ms;
        isValid = true;
        increment = 0.005;
        basePercentage = 0.965;
        
        if (QorS == 'Q' || QorS == 'q')
            gammaToReturn = 1;
            while ((rPeakLoc-gammaToReturn)/fs > timeWindowToLookFor)
                basePercentage = basePercentage + increment;
                thresholdForGamma = pi/2*basePercentage;
                posGammaLocs = phaseSeg(1:rPeakLoc) <= thresholdForGamma;
                if (all(posGammaLocs))
                    isValid = false;
                    break;
                end
                gammaToReturn = find(posGammaLocs, 1, 'last');
            end
        else
            gammaToReturn = length(phaseSeg);
            while (gammaToReturn/fs > timeWindowToLookFor)
                basePercentage = basePercentage + increment;
                thresholdForGamma = pi/2*basePercentage;
                posGammaLocs = phaseSeg(rPeakLoc:end) <= thresholdForGamma;
                if (all(posGammaLocs))
                    isValid = false;
                    break;
                end
                gammaToReturn = find(posGammaLocs, 1);
            end
        end
        
    end

end