function [rPeaksInd] = BurakSegtoRPeaks(phaseSignal, ampSignal, signal,fs, isFigure)
%From the phasor transform, does R peak detection.
%phaseSignal: Phase signal of the phasor transform.
%ampSignal: Amplitude signal of the phasor transofrm.
%rPeaksInd: Indices of the R-peaks.
%sign: 1 if R-peaks are valid, 0 if R-peaks are most likely invalid.
%% ECG signal thresholding
threshold = 0.003;
QRSSegments = phaseSignal > (pi/2 - threshold);
%% QRS Segments to R-peaks and R-peaks to QRS boundaries
onOffSetIndex = find(diff(QRSSegments));
if (isempty(onOffSetIndex) || max(abs(signal)) < 0.8)
    signal = signal./max(abs(signal));
    [phaseSignal, ampSignal] = BurakPTransform(signal, 0.001);
    threshold = 0.003;
    QRSSegments = phaseSignal > (pi/2 - threshold);
    onOffSetIndex = find(diff(QRSSegments));
end
if (isempty(onOffSetIndex))
   rPeaksInd = 0;
   return;
end
if (QRSSegments(1) ~= 1)
    numRpeaks = ceil(length(onOffSetIndex)/2);
else
    numRpeaks = ceil((length(onOffSetIndex) + 1)/2);
end
rPeaksInd = zeros(numRpeaks,1);
RToQRSBoundaries = zeros(numRpeaks,2);
iter = 1;
if (QRSSegments(1) == 1)
    [~, rPeaksInd(1)] = max(ampSignal(1:onOffSetIndex(1)));
    RToQRSBoundaries(1,:) = [1, onOffSetIndex(1)];
    onOffSetIndex(1) = [];
    iter = 2;
end
for l = 1:2:length(onOffSetIndex)
    leftIndex = onOffSetIndex(l) + 1;
    if (l + 1 <= length(onOffSetIndex))
        rightIndex = onOffSetIndex(l+1);
    else
        rightIndex = length(ampSignal);
    end
    [~, localOpt] = max(ampSignal(leftIndex:rightIndex));
    rPeaksInd(iter) = leftIndex + localOpt - 1;
    RToQRSBoundaries(iter,:) = [leftIndex, rightIndex];
    iter = iter + 1;
end

%% If the bpm is < 30, decleare an erroneous waveform.
% mins = length(signal)/fs/60;
% bpm = (length(rPeaksInd)-1)/mins;
% if (bpm < 20)
%     rPeaksInd = 0;
%     return
% end

%% If the first beat is erroneous, fix it.
while (length(rPeaksInd) >= 2 && (rPeaksInd(2)-rPeaksInd(1))/fs < 250e-3)
    leftR = phaseSignal(rPeaksInd(1));
    rightR = phaseSignal(rPeaksInd(2));
    if (leftR > rightR)
        rPeaksInd(2) = [];
        RToQRSBoundaries(2,:) = [];
    else
        rPeaksInd(1) = [];
        RToQRSBoundaries(1,:) = [];
    end
end

%% If the bpm is < 30, decleare an erroneous waveform.
mins = length(signal)/fs/60;
bpm = (length(rPeaksInd)-1)/mins;
rrInterval = diff(rPeaksInd);
if (bpm < 15 || any(rrInterval > 2.6*fs) || rPeaksInd(1) > 2.6*fs ||...
        (length(signal)-rPeaksInd(end)) > 2.6*fs)
    rPeaksInd = 0;
    return
end

%% R peak correction based on the paper
newRPeaksInd = BurakCorrectRPks(rPeaksInd, RToQRSBoundaries, phaseSignal, ampSignal, signal, fs);
if (isFigure)
    %     figure(fHandle(1))
    %     xAxis = 1:length(signal);
    %     subplot(2,1,1)
    %     plot(signal)
    %     hold on
    %     plot(rPeaksInd, signal(rPeaksInd), 'ro')
    %     %     falsePositiveRPeaks = rPeaksInd(2:end);
    %     %     falseNegativeRPeaks = rPeaksInd(2:end);
    %     %     RRInterval = diff(rPeakInds);
    %     %     falseNegativeRPeaks = falseNegativeRPeaks(falseNegatives) + RRInterval(falseNegatives)./2;
    %     %     falsePositiveRPeaks = falsePositiveRPeaks(falsePositives);
    %     %     yy = ones(length(falseNegativeRPeaks),1).*max(signal);
    %     %     plot(falseNegativeRPeaks, yy, 'ko')
    %     %     plot(falsePositiveRPeaks, signal(falsePositiveRPeaks), 'mo')
    %     hold off
    %     subplot(2,1,2)
    %     plot(phaseSignal)
    %     hold on
    %     plot(xAxis(QRSSegments), phaseSignal(QRSSegments), 'ro')
    %     hold off
    figure(fHandle(1))
    plot(signal)
    hold on
    plot(rPeaksInd, signal(rPeaksInd) + 2, 'ko')
    plot(newRPeaksInd, signal(newRPeaksInd), 'ro')
    %     plot(rPeaksInd, signal(rPeaksInd), 'ko')
    hold off
end
rPeaksInd = newRPeaksInd;
mins = length(signal)/fs/60;
bpm = (length(rPeaksInd)-1)/mins;
rrInterval = diff(rPeaksInd);
if (bpm < 15 || bpm > 260 || any(rrInterval > 2.6*fs) || rPeaksInd(1) > 2.6*fs ||...
        (length(signal)-rPeaksInd(end)) > 2.6*fs)
    rPeaksInd = 0;
    return
end
% falsePositives = findrrIntervals(2:end) < rrIntervals(1:end-1).*0.4;
end